@@ -121,7 +121,7 @@ error_out:
 static void
 create_directories(zathura_t* zathura)
 {
-  static const unsigned int mode = 0711;
+  static const unsigned int mode = 0700;
 
   if (g_mkdir_with_parents(zathura->config.config_dir, mode) == -1) {
     girara_error("Could not create '%s': %s", zathura->config.config_dir,
@@ -751,7 +751,7 @@ document_open_password_dialog(gpointer data)
   zathura_password_dialog_info_t* password_dialog_info = data;
 
   girara_dialog(password_dialog_info->zathura->ui.session, _("Enter password:"), true, NULL,
-                (girara_callback_inputbar_activate_t) cb_password_dialog, password_dialog_info);
+                cb_password_dialog, password_dialog_info);
   return FALSE;
 }
 
@@ -763,7 +763,6 @@ document_open(zathura_t* zathura, const char* path, const char* uri, const char*
     goto error_out;
   }
 
-  gchar* file_uri = NULL;
   zathura_error_t error = ZATHURA_ERROR_OK;
   zathura_document_t* document = zathura_document_open(zathura, path, uri, password, &error);
 
@@ -882,31 +881,22 @@ document_open(zathura_t* zathura, const char* path, const char* uri, const char*
   g_free(filename);
 
   /* install file monitor */
-  file_uri = g_filename_to_uri(file_path, NULL, NULL);
-  if (file_uri == NULL) {
-    goto error_free;
-  }
-
-  if (zathura->file_monitor.file == NULL) {
-    zathura->file_monitor.file = g_file_new_for_uri(file_uri);
-    if (zathura->file_monitor.file == NULL) {
-      goto error_free;
-    }
-  }
-
   if (zathura->file_monitor.monitor == NULL) {
-    zathura->file_monitor.monitor = g_file_monitor_file(zathura->file_monitor.file, G_FILE_MONITOR_NONE, NULL, NULL);
+    char* filemonitor_backend = NULL;
+    girara_setting_get(zathura->ui.session, "filemonitor", &filemonitor_backend);
+    zathura_filemonitor_type_t type = ZATHURA_FILEMONITOR_GLIB;
+#ifdef G_OS_UNIX
+    if (g_strcmp0(filemonitor_backend, "signal") == 0) {
+      type = ZATHURA_FILEMONITOR_SIGNAL;
+    }
+#endif
+
+    zathura->file_monitor.monitor = zathura_filemonitor_new(file_path, type);
     if (zathura->file_monitor.monitor == NULL) {
       goto error_free;
     }
-    g_signal_connect(G_OBJECT(zathura->file_monitor.monitor), "changed", G_CALLBACK(cb_file_monitor), zathura->ui.session);
-  }
-
-  if (zathura->file_monitor.file_path == NULL) {
-    zathura->file_monitor.file_path = g_strdup(file_path);
-    if (zathura->file_monitor.file_path == NULL) {
-      goto error_free;
-    }
+    g_signal_connect(G_OBJECT(zathura->file_monitor.monitor), "reload-file",
+                     G_CALLBACK(cb_file_monitor), zathura->ui.session);
   }
 
   if (password != NULL) {
@@ -1017,7 +1007,7 @@ document_open(zathura_t* zathura, const char* path, const char* uri, const char*
   }
 
   /* read first_page_column list */
-  if (file_info.first_page_column_list != NULL && strcmp(file_info.first_page_column_list, "")) {
+  if (file_info.first_page_column_list != NULL && *file_info.first_page_column_list != '\0') {
     first_page_column_list = file_info.first_page_column_list;
     file_info.first_page_column_list = NULL;
   } else {
@@ -1055,8 +1045,6 @@ document_open(zathura_t* zathura, const char* path, const char* uri, const char*
   girara_set_window_title(zathura->ui.session, formatted_filename);
   g_free(formatted_filename);
 
-  g_free(file_uri);
-
   /* adjust_view */
   adjust_view(zathura);
   for (unsigned int page_id = 0; page_id < number_of_pages; page_id++) {
@@ -1092,10 +1080,6 @@ document_open(zathura_t* zathura, const char* path, const char* uri, const char*
 
 error_free:
 
-  if (file_uri != NULL) {
-    g_free(file_uri);
-  }
-
   zathura_document_free(document);
 
 error_out:
@@ -1168,7 +1152,7 @@ document_save(zathura_t* zathura, const char* path, bool overwrite)
   if (g_file_test(file_path, G_FILE_TEST_IS_DIR) == TRUE) {
     char* basename = g_path_get_basename(zathura_document_get_path(zathura->document));
     char* tmp = file_path;
-    file_path = g_strconcat(file_path, "/", basename, NULL);
+    file_path = g_build_filename(file_path, basename, NULL);
     g_free(tmp);
     g_free(basename);
   }
@@ -1233,21 +1217,10 @@ document_close(zathura_t* zathura, bool keep_monitor)
   /* remove monitor */
   if (keep_monitor == false) {
     if (zathura->file_monitor.monitor != NULL) {
-      g_file_monitor_cancel(zathura->file_monitor.monitor);
       g_object_unref(zathura->file_monitor.monitor);
       zathura->file_monitor.monitor = NULL;
     }
 
-    if (zathura->file_monitor.file != NULL) {
-      g_object_unref(zathura->file_monitor.file);
-      zathura->file_monitor.file = NULL;
-    }
-
-    if (zathura->file_monitor.file_path != NULL) {
-      g_free(zathura->file_monitor.file_path);
-      zathura->file_monitor.file_path = NULL;
-    }
-
     if (zathura->file_monitor.password != NULL) {
       g_free(zathura->file_monitor.password);
       zathura->file_monitor.password = NULL;
@@ -1509,9 +1482,8 @@ adjust_view(zathura_t* zathura)
       abs((int)new_cell_height - (int)cell_height) > 1) {
     render_all(zathura);
     refresh_view(zathura);
-
- /* otherwise set the old scale and leave */
   } else {
+    /* otherwise set the old scale and leave */
     zathura_document_set_scale(zathura->document, scale);
   }
 
